"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildDynamoUpdateExpression = exports.buildDynamoUpdateExpressionForType = exports.buildUpdateExpressionAttributeValues = exports.buildUpdateExpressionAttributeNames = void 0;
const quidproquo_core_1 = require("quidproquo-core");
const buildDynamoQuery_1 = require("./buildDynamoQuery");
const buildUpdateExpressionAttributeNames = (updates) => {
    let attributeNames = {};
    for (let update of updates) {
        if (Array.isArray(update.attributePath)) {
            // Handle nested attributes
            for (let i = 0; i < update.attributePath.length; i++) {
                let attribute = update.attributePath[i];
                if (typeof attribute === 'string') {
                    const itemName = (0, buildDynamoQuery_1.getItemName)(attribute);
                    attributeNames[itemName] = attribute;
                    // Replace attribute name in path with safe name
                    update.attributePath[i] = itemName;
                }
            }
        }
        else {
            // Handle top-level attributes
            const itemName = (0, buildDynamoQuery_1.getItemName)(update.attributePath);
            attributeNames[itemName] = update.attributePath;
            update.attributePath = itemName;
        }
    }
    return attributeNames;
};
exports.buildUpdateExpressionAttributeNames = buildUpdateExpressionAttributeNames;
const buildUpdateExpressionAttributeValues = (updates) => {
    let attributeValues = {};
    for (let update of updates) {
        if (update.value !== undefined && update.value !== null) {
            const valuePlaceholder = (0, buildDynamoQuery_1.getValueName)(update.value);
            attributeValues[valuePlaceholder] = (0, buildDynamoQuery_1.buildAttributeValue)(update.value);
        }
    }
    return Object.keys(attributeValues).length > 0 ? attributeValues : undefined;
};
exports.buildUpdateExpressionAttributeValues = buildUpdateExpressionAttributeValues;
const buildDynamoUpdateExpressionSet = (update) => {
    if (update.value === undefined || update.value === null) {
        throw new Error("Value must be provided for 'SET' action");
    }
    return `${getNestedItemName(update.attributePath)} = ${(0, buildDynamoQuery_1.getValueName)(update.value)}`;
};
const buildDynamoUpdateExpressionRemove = (update) => {
    return `${getNestedItemName(update.attributePath)}`;
};
const buildDynamoUpdateExpressionAdd = (update) => {
    if (update.value === undefined || update.value === null) {
        throw new Error("Value must be provided for 'ADD' action");
    }
    return `${getNestedItemName(update.attributePath)} ${(0, buildDynamoQuery_1.getValueName)(update.value)}`;
};
const buildDynamoUpdateExpressionDelete = (update) => {
    if (update.value !== undefined && update.value !== null) {
        return `${getNestedItemName(update.attributePath)} ${(0, buildDynamoQuery_1.getValueName)(update.value)}`;
    }
    else {
        return `${getNestedItemName(update.attributePath)}`;
    }
};
const getNestedItemName = (attributePath) => {
    if (Array.isArray(attributePath)) {
        let path = '';
        for (let i = 0; i < attributePath.length; i++) {
            if (typeof attributePath[i] === 'string') {
                path += `.${(0, buildDynamoQuery_1.getItemName)(attributePath[i])}`;
            }
            else {
                path += `[${attributePath[i]}]`;
            }
        }
        return path.substring(1); // remove the leading dot
    }
    else {
        return (0, buildDynamoQuery_1.getItemName)(attributePath);
    }
};
const buildDynamoUpdateExpressionPart = (update, updateIndex) => {
    switch (update.action) {
        case quidproquo_core_1.KvsUpdateActionType.Set:
            return buildDynamoUpdateExpressionSet(update);
        case quidproquo_core_1.KvsUpdateActionType.Remove:
            return buildDynamoUpdateExpressionRemove(update);
        case quidproquo_core_1.KvsUpdateActionType.Add:
            return buildDynamoUpdateExpressionAdd(update);
        case quidproquo_core_1.KvsUpdateActionType.Delete:
            return buildDynamoUpdateExpressionDelete(update);
        default:
            throw new Error(`Invalid update action type: ${update.action}`);
    }
};
const buildDynamoUpdateExpressionForType = (type, kvsUpdate) => {
    const actions = kvsUpdate.filter((update) => update.action === type);
    // If there are no actions of this type, return an empty string
    if (actions.length === 0) {
        return '';
    }
    const expressions = actions
        .map((update, index) => buildDynamoUpdateExpressionPart(update, index))
        .join(', ');
    switch (type) {
        case quidproquo_core_1.KvsUpdateActionType.Set:
            return `SET ${expressions}`;
        case quidproquo_core_1.KvsUpdateActionType.Remove:
            return `REMOVE ${expressions}`;
        case quidproquo_core_1.KvsUpdateActionType.Add:
            return `ADD ${expressions}`;
        case quidproquo_core_1.KvsUpdateActionType.Delete:
            return `DELETE ${expressions}`;
        default:
            throw new Error(`Invalid update action type: ${type}`);
    }
};
exports.buildDynamoUpdateExpressionForType = buildDynamoUpdateExpressionForType;
const buildDynamoUpdateExpression = (updates) => {
    const updatesExpressions = [
        quidproquo_core_1.KvsUpdateActionType.Set,
        quidproquo_core_1.KvsUpdateActionType.Remove,
        quidproquo_core_1.KvsUpdateActionType.Add,
        quidproquo_core_1.KvsUpdateActionType.Delete,
    ]
        .map((kvsUpdateActionType) => (0, exports.buildDynamoUpdateExpressionForType)(kvsUpdateActionType, updates))
        .filter((expression) => !!expression);
    const result = updatesExpressions.join(' ');
    console.log('Update Expression: ', result);
    return result;
};
exports.buildDynamoUpdateExpression = buildDynamoUpdateExpression;
